/*
*****************************************************************************
*            Windows-API for Transponder-Reader Family T4/T5                *
*****************************************************************************
*       Scope:  Function Prototypes for DLL T4W.DLL/T4CE.DLL                *
*        File:  T4W.H                                                       *
*     Version:  1.50                                                        *
*        Date:  03APR09                                                     *
*****************************************************************************
* H I S T O R Y                                                             *
*                                                                           *
* 1.00 04APR97  Initial release.                                            *
* 1.01 11APR97  T4_SelectTag and T4_SelectTagPW ary trying to select        *
*               the tag one more time if no tag is found.                   *
* 1.02 16APR97  New function 'T4SetTagOptions' to set options, how tags     *
*               are processed by the tag reader.                            *
* 1.03 12JUN97  New function 'T4SetAPWM' to switch on and off               *
*               autopasswordmode.                                           *
* 1.04 04JUL97  New function 'T4InitPollTag' and 'T4DoPollTag' added.       *
* 1.10 19FEB97  New function 'T4SetTagDriver' added.                        *
* 1.11 05JUL99  Return of serial number, even if password is wrong.         *
*               Error code is left unchanged.                               *
* 1.12 26APR00  Init-Lock, Errorcodes, therefor 'T4InitPollTag' and         *
*               'T4DeInitPollTag' are also returning error codes instead of *
*               void.                                                       *
* 1.13 26SEP00  New/changed error codes.                                    *
*               Public routine 'T4GetStatusString'                          *
* 1.14 21JUN01  New definitions for IDRW-E                                  *
* 1.15 02AUG01  Many new Functions:                                         *
*               T4CPUReset, T4ConfigurationReset, T4RFReset, T4IOSetConfig, *
*               T4IOSetDefaults, T4AAMSetConfig, T4AAMSetRepeatTime,        *
*               T4IOReadInputs, T4IOSetOutputs, T4IOClearOutputs, T4IOBeep, *
*               T4ReaderOn, T4ReaderOff                                     *
*               Support for Auto Answer Mode, General Purpos I/O            *
* 1.16 22OCT01  Release Version                                             *
* 1.18 22JAN02  Release Version, common source for Windows and Windows CE   *
*               New CPU supported: Windows CE ARM                           *
* 1.19 25JAN02  Increased wait time within T4Init                           *
* 1.20 25FEB02  New function T4SelectTagType                                *
* 1.21 28MAR02  Support for multiple transponder raeders                    *
* 1.22 08MAY02  New function for calculating IDROA raw data                 *
* 1.23 17JUN02  CE version compiled under H/PC Pro 2.11                     *
* 1.24 10JUL02  Solved init problem within T4Init                           *
* 1.25 13JAN03  T4FormatTagReader optimzed for compatibility with           *
*               with 3rd party readers                                      *
* 1.30 21MAY03  New function T4FindCOMPort for CF card reader and special   *
*               new behaviour on removing hardware                          *
* 1.32 12DEC03  New function T4ChangeReaderPW                               *
*               Improved PnP behaviour                                      *
*               DLL for PC is now returning T4_V24_NOTINIT, too, if the     *
*               reader has been removed (CF card/USB)                       *
*               New fucntion T4FindCOMPortNr for CF card readers which      *
*               returns the number of a COM port as a integer               *
*               New function T4GetCOMPortString which creates a COM port    *
*               string out of a port number                                 *
* 1.33 16DEC03  Compilation without standard libs (saving space)            *
* 1.34 17JAN04  Support for OEM version of CF card reader                   *
*               New function T4GetStatusString2 for easier VB support       *
* 1.35 22MAR04  T4FindCOMPort... now available for use under Win2000/XP     *
* 1.37 10MAY04  Bug fix regarding IDRO-G transponders                       *
* 1.38 26MAY04  New function T4GetTagPollState                              *
* 1.39 23JUN04  Support for firmware update (needs firmware V2.61)          *
* 1.40 07FEB05  T5 specific extensions                                      *
* 1.41 14MAR05  Telegram specific timeout                                   *
* 1.42 11AUG05  Support for IDRO-H                                          * 
* 1.43 23SEP05  Adapated transmission routine for fast PDA's                *
*               Increased inter-character timeout                           *
* 1.44 20DEC05  New function T4GetHandle                                    *
*               Removed inter-character timeout completely                  *
* 1.45 31JAN06  Improved re-synchronization                                 *
*      29JAN07  Definitions for transponder readers W/ crypto processor     *
* 1.46 13NOV07  New command SetRFPower                                      *
* 1.48 24JUN08  Support for TWN3 reader                                     *
* 1.49 24SEP08  Support for Windows Mobile 6                                *
* 1.50 03APR09  OEM Version                                                 *
*****************************************************************************
*/

/* TAB 3 */

#ifndef  __T4W_H__
#define  __T4W_H__

/****** Allgemein *****************************************/

#ifdef __LIBMAKE__
    #define LIBFUNC     __declspec(dllexport) WINAPI
    #define EXPFUNC     LIBFUNC
#else
    #define LIBFUNC     WINAPI
#endif

#define T4_VERSIONLIBRARY     150

/****** Fehlercodes ***************************************/

#define T4_NOERROR            0x00

#define T4_NOTAG              0x01
#define T4_DATAERROR          0x02
#define T4_WRITEERROR         0x03
#define T4_ADRESSERROR        0x04
#define T4_WRONGTAG           0x05
#define T4_READERROR          0x06
#define T4_NOTSELECTED        0x07
#define T4_WRONGTAGPW         0x08

#define T4_PARAERRWRITE       0x10
#define T4_PARANOTVALID       0x11
#define T4_PARAERRPWCYCLE     0x12
#define T4_PARANOLOGIN        0x13
#define T4_PARAWRONGPW        0x14

#define T4_READERCHPWFAIL     0x60

#define T4_UNKNOWNCOMMAND     0x80
#define T4_WRONGLENGTH        0x81
#define T4_CMDNOTAVAILABLE    0x82
#define T4_COMERRORRFMODULE   0x83

#define T4_RX_WRONGLENGTH     0x93
#define T4_WRONGBLOCKCNT      0x95
#define T4_POLLNOTINIT        0x96
#define T4_POLLIDLE           0x97
#define T4_POLLBUSY           0x98

#define T4_ALREADYINIT        0xA0
#define T4_NOTINIT            0xA1
#define T4_V24_INITFAIL       0xA2
#define T4_NOREADERSELECTED   0xA4

#define T4_RX_TIMEOUT         0xB1
#define T4_RX_WRONGANSWER     0xB2
#define T4_RX_WRONGCSUM       0xB3
#define T4_RX_PARITY          0xB4
#define T4_RX_OVERRUN         0xB5
#define T4_RX_FRAME           0xB6

/****** Definition der Transpondertypen *******************/

#define T4_TYPE_IDRO_A        0x00
#define T4_TYPE_IDRW_B        0x01
#define T4_TYPE_IDRW_C        0x02
#define T4_TYPE_IDRW_D        0x03
#define T4_TYPE_IDRW_E        0x04
#define T4_TYPE_IDRO_G        0x05
#define T4_TYPE_IDRO_H        0x06

#define T4_MSK_IDRO_A         0x01
#define T4_MSK_IDRW_B         0x02
#define T4_MSK_IDRW_C         0x04
#define T4_MSK_IDRW_D         0x08
#define T4_MSK_IDRW_E         0x10
#define T4_MSK_IDRO_G         0x20
#define T4_MSK_IDRO_H         0x40

/****** Definition of Port Types **************************/

#define T4_MSK_USB            0x01
#define T4_MSK_CF             0x02

/****** Definitions for Crypto Modes **********************/

#define HTG1_AUTH_NONE        0x00
#define HTG1_AUTH_KEYA        0x01
#define HTG1_AUTH_KEYB        0x02

#define HTGS_AUTH_NONE        0x00
#define HTGS_AUTH_STD         0x10
#define HTGS_AUTH_PW          0x20
#define HTGS_AUTH_NONESTD     0x30
#define HTGS_AUTH_NONEPW      0x40
 
#define HTG2_AUTH_PW          0x00
#define HTG2_AUTH_CRYPT       0x01
#define HTG2_AUTH_CRYPT_PW    0x02
 
/**********************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/****** DLL Functions *************************************/

int  LIBFUNC T4Init(LPCTSTR Port);
int  LIBFUNC T4DeInit(void);

LPCTSTR LIBFUNC T4FindCOMPort(void);
int LIBFUNC T4FindCOMPortNr(void);

#ifndef UNDER_CE
int LIBFUNC T4FindFirstCOMPortNr(int PortTypes);
int LIBFUNC T4FindNextCOMPortNr();
#endif

HANDLE LIBFUNC T4GetHandle(void);
HANDLE LIBFUNC T4GetCurrentReader(void);
void LIBFUNC T4SetCurrentReader(HANDLE Handle);

int  LIBFUNC T4VersionLibrary(void);
LPCTSTR LIBFUNC T4GetStatusString(int Status);
void LIBFUNC T4GetStatusString2(int Status,LPTSTR StatusString);

/****** General Purpose Functions *************************/

int  LIBFUNC T4CPUReset(void);
int  LIBFUNC T4ConfigurationReset(void);
int  LIBFUNC T4VersionTagReader(BYTE *Version);
int  LIBFUNC T4RFReset(void);
int  LIBFUNC T4Halt(void);

int  LIBFUNC T4ReaderOn(void);
int  LIBFUNC T4ReaderOff(void);

/****** Reader Configuration ******************************/

/*----- Configure Tag Handling ---------------------------*/

int  LIBFUNC T4SetTagDriver(BYTE Driver);
int  LIBFUNC T4SetTagOptions(BYTE TagType,BYTE TagOptions);

/*----- Configure IO -------------------------------------*/

int  LIBFUNC T4IOSetConfig(BYTE IODirection,BYTE IOLevel);
int  LIBFUNC T4IOSetDefaults(BYTE IODefaultValue);

/*----- Configure Auto Answer Mode -----------------------*/

int  LIBFUNC T4AAMSetConfig(BYTE AAMMode);
int  LIBFUNC T4AAMSetRepeatTime(WORD RepeatTime);

/*----- Configure Internal Reader Password ---------------*/

int  LIBFUNC T4ChangeReaderPW(BYTE *OldPW,BYTE *NewPW);

/*---- Crypto Configuration ------------------------------*/

int  LIBFUNC T4Personalisation12(const BYTE *Data);
int  LIBFUNC T4PersonalisationS(const BYTE *Data);

/*---- Setup RF Power ------------------------------------*/

int  LIBFUNC T4SetRFPower(const BYTE RFPower);

/****** Transponder Functions *****************************/

/* Select any configured transponder, use internal password from the reader */
int  LIBFUNC T4SelectTag(BYTE *TagType,BYTE *SerialNumber);
/* Select any configured transponder, password is given */
int  LIBFUNC T4SelectTagPW(BYTE *PassWord,BYTE *TagType,BYTE *SerialNumber);
/* Select a specific transponder, use internal password from the reader */
int  LIBFUNC T4SelectTagType(BYTE ReqTagType,BYTE *TagType,BYTE *SerialNumber);

int  LIBFUNC T4InitTagPoll(BYTE *PassWord);
int  LIBFUNC T4DeInitTagPoll(void);
int  LIBFUNC T4GetTagPollState(void);
int  LIBFUNC T4DoTagPoll(BYTE *TagType,BYTE *SerialNumber);

int  LIBFUNC T4InitAAMPoll(void);
int  LIBFUNC T4DeInitAAMPoll(void);
int  LIBFUNC T4DoAAMPoll(BYTE *TagType,BYTE *SerialNumber);

int  LIBFUNC T4ReadData(BYTE BlockAdress,BYTE BlockCount,BYTE *Data);
int  LIBFUNC T4WriteData(BYTE BlockAdress,BYTE BlockCount,BYTE *Data);

int  LIBFUNC T4ReadConfigData(BYTE ConfigAdress,BYTE *ConfigData);
int  LIBFUNC T4WriteConfigData(BYTE ConfigAdress,BYTE *ConfigData);

/****** I/O Functions *************************************/

int  LIBFUNC T4IOReadInputs(BYTE *InputState);
int  LIBFUNC T4IOSetOutputs(BYTE SetBits);
int  LIBFUNC T4IOClearOutputs(BYTE ClearBits);
int  LIBFUNC T4IOBeep(BYTE BeepBits,BYTE Duration,BYTE Frequency);

/****** Useful Helper Functions ***************************/

void LIBFUNC T4CopyBit(BYTE *Src,BYTE *Dst,int BitSrc,int BitDst);
void LIBFUNC T4CopyBits(BYTE *Src,BYTE *Dst,int BitSrc,int BitDst,int BitCnt,bool Reverse);
LPCTSTR LIBFUNC T4GetDecString(BYTE *Data,int Bits);
LPCTSTR LIBFUNC T4GetCOMPortString(int PortNr);
int  LIBFUNC T4CalcIDROARawData(BYTE *TagID,BYTE *RawTagID);

/**********************************************************/

#ifdef __cplusplus
}
#endif

#endif
